import json

from jsonschema import RefResolver

from .schema import Schema

BOUNTY_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "bounty",
    "definitions": {
        "file": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "filename": {
                        "type": [
                            "string",
                            "null"
                        ]
                    },
                    "filesize": {
                        "type": [
                            "string",
                            "null"
                        ]
                    },
                    "mimetype": {
                        "type": "string"
                    },
                    "sha256": {
                        "type": [
                            "string",
                            "null"
                        ]
                    },
                    "sha1": {
                        "type": [
                            "string",
                            "null"
                        ]
                    },
                    "md5": {
                        "type": [
                            "string",
                            "null"
                        ]
                    }
                },
                "additionalProperties": True,
                "required": [
                    "mimetype"
                ]
            }
        },
        "url": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "protocol": {"type": "string"}
                },
                "additionalProperties": True,
                "required": ["protocol"]
            }
        }
    },
    "type": "array",
    "oneOf": [
            {
                "$ref": "#/definitions/file"
            },
            {
                "$ref": "#/definitions/url"
            }
        ]
}


class Bounty(Schema):
    def __init__(self, ):
        self.artifacts = []

    def add_file_artifact(self, mimetype, filename=None, filesize=None, sha256=None, sha1=None, md5=None):
        if mimetype is None:
            raise ValueError("Mimetype cannot be None")
        artifact = {
            "mimetype": mimetype,
        }
        if filename is not None:
            artifact['filename'] = filename

        if filesize is not None:
            artifact['filesize'] = filesize

        if sha256 is not None:
            artifact['sha256'] = sha256

        if sha1 is not None:
            artifact['sha1'] = sha1

        if md5 is not None:
            artifact['md5'] = md5

        self.artifacts.append(artifact)
        return self

    def add_url_artifact(self, *_args, **kwargs):
        self.artifacts.append({**kwargs})
        return self

    @classmethod
    def get_schema(cls):
        """
        Get the path to the backing schema this Metadata object users
        :return: Tuple[string, string] where first string is the path,
        and the second is the schema name
        """
        return BOUNTY_SCHEMA

    def json(self):
        """
        Convert metadata implementation into json string
        :return: JSON string representing the internal type of this object
        """
        output = BountyEncoder().encode(self)
        if not Bounty.validate(json.loads(output)):
            raise ValueError('Invalid Bounty setup')

        return output

    @classmethod
    def validate(cls, value, resolver=None, silent=False):
        resolver = RefResolver.from_schema(BOUNTY_SCHEMA)
        return super().validate(value, resolver, silent)


class BountyEncoder(json.JSONEncoder):
    def encode(self, obj):
        if isinstance(obj, Bounty):
            return json.dumps([artifact for artifact in obj.artifacts])



import json
import os
import pkg_resources
from jsonschema import RefResolver

from .schema import Schema
from .verdict import VerdictEncoder, VERDICT_SCHEMA

ASSERTION_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "assertion",
    "type": "array",
    "minItems": 1,
    "maxItems": 256,
    "items": {"$ref": "verdict"}
}


class Assertion(Schema):
    def __init__(self, ):
        self.artifacts = []

    def add_artifact(self, artifact):
        self.artifacts.append(artifact)
        return self

    def add_artifacts(self, artifacts):
        self.artifacts.extend(artifacts)
        return self

    @classmethod
    def get_schema(cls):
        """
        Get the path to the backing schema this Metadata object users
        :return: Tuple[string, string] where first string is the path,
        and the second is the schema name
        """
        return ASSERTION_SCHEMA

    @classmethod
    def validate(cls, value, resolver=None, silent=False):
        resolver = RefResolver.from_schema(VERDICT_SCHEMA)
        return super().validate(value, resolver, silent)

    def json(self):
        """
        Convert metadata implementation into json string
        :return: JSON string representing the internal type of this object
        """
        if any([artifact is None for artifact in self.artifacts]):
            raise ValueError('Artifacts cannot be None')
        output = AssertionEncoder().encode(self)
        if not Assertion.validate(json.loads(output)):
            raise ValueError('Invalid Bounty setup')

        return output


class AssertionEncoder(json.JSONEncoder):
    def encode(self, obj):
        if isinstance(obj, Assertion):
            return json.dumps([json.loads(VerdictEncoder().encode(artifact)) for artifact in obj.artifacts])



import logging
from enum import Enum

from .exceptions import DecodeError

logger = logging.getLogger(__name__)


class ArtifactType(Enum):
    FILE = 0
    URL = 1

    @staticmethod
    def from_string(value):
        if value is not None:
            try:
                return ArtifactType[value.upper()]
            except KeyError:
                logger.critical('%s is not a supported artifact type', value)

    @staticmethod
    def to_string(artifact_type):
        return artifact_type.name.lower()

    def decode_content(self, content):
        if content is None:
            return None

        if self == ArtifactType.URL:
            try:
                return content.decode('utf-8')
            except UnicodeDecodeError:
                raise DecodeError('Error decoding URL')
        else:
            return content
